/* xml++.cc
 * libxml++ and this file are copyright (C) 2000 by Ari Johnson, and
 * are covered by the GNU Lesser General Public License, which should be
 * included with libxml++ as the file COPYING.
 */

#include "libxml++/parsers/saxparser.h"
#include <libxml/parserInternals.h> // for xmlCreateFileParserCtxt
#include <fstream> //For std::ifstream.
#include <cstdarg> //For va_list.
#include <iostream> //For cerr

namespace xmlpp {


SaxParser::SaxParser()
	: _context(0), _exception(0)
{
  xmlSAXHandler temp = {
    NULL,		// internalSubset
    NULL,		// isStandalone
    NULL,		// hasInternalSubset
    NULL,		// hasExternalSubset
    NULL,		// resolveEntity
    _callback_get_entity,	// getEntity
    NULL,		// entityDecl
    NULL,		// notationDecl
    NULL,		// attributeDecl
    NULL,		// elementDecl
    NULL,		// unparsedEntityDecl
    NULL,		// setDocumentLocator
    _callback_start_document,	// startDocument
    _callback_end_document,	// endDocument
    _callback_start_element,	// startElement
    _callback_end_element,	// endElement
    NULL,		// reference
    _callback_characters,	// characters
    NULL,		// ignorableWhitespace
    NULL,		// processingInstruction
    _callback_comment,		// comment
    _callback_warning,		// warning
    _callback_error,		// error
    _callback_fatal_error,	// fatalError
    NULL,		// getParameterEntity
    NULL,		// cdataBlock
    NULL		// externalSubset
  };
  _sax_handler = temp;
}

SaxParser::~SaxParser() {
  if(_context)
    xmlFreeParserCtxt(_context);
}

xmlEntityPtr SaxParser::on_get_entity(const xmlChar* name)
{
    return xmlGetPredefinedEntity(name);
}

void SaxParser::on_start_document()
{
}

void SaxParser::on_end_document()
{
}

void SaxParser::on_start_element(const std::string& nameame, const Element::AttributeMap& attributes)
{
}

void SaxParser::on_end_element(const std::string& name)
{
}

void SaxParser::on_characters(const std::string& text)
{
}

void SaxParser::on_comment(const std::string& text)
{
}

void SaxParser::on_warning(const std::string& text)
{
}

void SaxParser::on_error(const std::string& text)
{
}

void SaxParser::on_fatal_error(const std::string& text)
{
  throw parse_error("Fatal error: " + text);
}

void SaxParser::handleException(const exception& e)
{
  _exception = e.Clone();
  xmlStopParser(_context);
}

// implementation of this function is inspeared from SAX documentation by James Henstridge.
// (http://www.daa.com.au/~james/gnome/xml-sax/implementing.html)
void SaxParser::parse()
{
  if(!_context)
    throw internal_error("Parse context not created.");

  _context->sax = & _sax_handler;
  _context->userData = this;

  xmlKeepBlanksDefault(0 /*FALSE*/); //Don't use libxml1 compatability. This is the default anyway.


  xmlParseDocument(_context);

  if( (! _context->wellFormed)
      && (! _exception) )
    _exception = new parse_error("Document not well-formed");

  _context->sax = NULL;
  xmlFreeParserCtxt(_context);
  _context = NULL;

  if(_exception)
  {
    exception * tmp = _exception;
    _exception = 0;
    tmp->Raise();
  }
}

void SaxParser::parse_file(const std::string& filename) throw(exception)
{
  if(_context)
    throw parse_error("Attempt to start a second parse while a parse is in progress.");

  _context = xmlCreateFileParserCtxt(filename.c_str());
  parse();
}

void SaxParser::parse_memory(const std::string& contents) throw(exception)
{
  if(_context)
    throw parse_error("Attempt to start a second parse while a parse is in progress.");

  _context = xmlCreateMemoryParserCtxt(contents.c_str(), contents.length());
  parse();
}

void SaxParser::parse_stream(std::istream& in) throw(exception)
{
  if(_context)
    throw parse_error("Attempt to start a second parse while a parse is in progress.");


  std::string line;

  
  _context = xmlCreatePushParserCtxt(
      &_sax_handler,
      this, // user_data
      NULL,
      0,
      ""); // here should come the filename. I don't know if it is a problem to let it empty

  while( ( ! _exception )
      && getline(in, line))
  {
    // since getline does not get the line separator, we have to add it since the parser care
    // about layout in certain cases.
    line += '\n';

    xmlParseChunk(_context, line.c_str(), line.length(), 0);
  }

  if( ! _exception )
    xmlParseChunk(_context, NULL, 0, 1);

  xmlFreeParserCtxt(_context);
  _context = NULL;

  if( _exception )
  {
    exception * tmp = _exception;
    _exception = 0;
    tmp->Raise();
  }
}

xmlEntityPtr SaxParser::_callback_get_entity(void *_parser, const xmlChar *name) {
  SaxParser* parser = static_cast<SaxParser*>(_parser);
  return parser->on_get_entity(name);
}

void SaxParser::_callback_start_document(void *_parser) {
  SaxParser *parser = (SaxParser *) _parser;
  try {
    parser->on_start_document();
  } catch (const exception& e) {
    parser->handleException(e);
  }
};

void SaxParser::_callback_end_document(void *_parser) {
  SaxParser *parser = (SaxParser *) _parser;
  if(parser->_exception)
    return;

  try {
    parser->on_end_document();
  } catch (const exception& e) {
    parser->handleException(e);
  }
};

void SaxParser::_callback_start_element(void *_parser, const xmlChar *name, const xmlChar **p) {
  SaxParser *parser = (SaxParser *) _parser;
  Element::AttributeMap attributes;

  if(p) {
    const xmlChar **cur;

    for(cur = p; cur && *cur; cur++) {
      std::string name, value;

      name = std::string((const char *) *cur++);
      value = std::string((const char *) *cur);

      attributes[name] = new Attribute(name, value);
    }
  }

  try {
    parser->on_start_element(std::string((const char *) name), attributes);
  } catch (const exception& e) {
    parser->handleException(e);
  }

  for(Element::AttributeMap::iterator i = attributes.begin();
      i != attributes.end();
      ++i)
  {
    delete i->second;
  }
};

void SaxParser::_callback_end_element(void *_parser, const xmlChar *name) {
  SaxParser *parser = (SaxParser *) _parser;
  try {
    parser->on_end_element(std::string((const char *) name));
  } catch (const exception& e) {
    parser->handleException(e);
  }
};

void SaxParser::_callback_characters(void *_parser, const xmlChar *ch, int len) {
  SaxParser *parser = (SaxParser *) _parser;
  try {
    parser->on_characters(std::string((const char *) ch, len));
  } catch (const exception& e) {
    parser->handleException(e);
  }
};

void SaxParser::_callback_comment(void *_parser, const xmlChar *value) {
  SaxParser *parser = (SaxParser *) _parser;
  try {
    parser->on_comment(std::string((const char *) value));
  } catch (const exception& e) {
    parser->handleException(e);
  }
};

void SaxParser::_callback_warning(void *_parser, const char *fmt, ...) {
  SaxParser *parser = (SaxParser *) _parser;

  va_list arg;
  char buff[1024];

  va_start(arg, fmt);
  vsprintf(buff, fmt, arg);
  va_end(arg);

  try {
    parser->on_warning(std::string(buff));
  } catch (const exception& e) {
    parser->handleException(e);
  }
};

void SaxParser::_callback_error(void *_parser, const char *fmt, ...) {
  SaxParser *parser = (SaxParser *) _parser;
  va_list arg;
  char buff[1024];

  if(parser->_exception)
    return;

  va_start(arg, fmt);
  vsprintf(buff, fmt, arg);
  va_end(arg);

  try {
    parser->on_error(std::string(buff));
  } catch (const exception& e) {
    parser->handleException(e);
  }
};

void SaxParser::_callback_fatal_error(void *_parser, const char *fmt, ...) {
  SaxParser *parser = (SaxParser *) _parser;
  va_list arg;
  char buff[1024];

  va_start(arg, fmt);
  vsprintf(buff, fmt, arg);
  va_end(arg);

  try {
    parser->on_fatal_error(std::string(buff));
  } catch (const exception& e) {
    parser->handleException(e);
  }
};


} // namespace xmlpp


